using System;
using System.Collections.Generic;
using System.Text;
using ceometric.ComputationalGeometry;
using ceometric.VectorGeometry;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// You must reference both V# and G# first to run this program
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
namespace MeshReduction
{
    class Program
    {
        static void Main(string[] args)
        {
            //Read some triangles from an .off format ASCII file
            OFFReader or = new OFFReader();
            or.Read(@"..\..\shark_en.off");

            //Build a mesh from the unordered triangles read from the .off file.
            //A mesh is a data structure comprising a list of vertices and a list
            //of mesh triangles whereby an mesh triangle is a list of three integer
            //values referring to the indices of the vertex list. 
            //The mesh also has fully referenced adjacency information. Vertices
            //know their incident triangles, edges know their adjacent triangles
            //and triangles know the edges they are composed of. 
            Mesh m = new Mesh();
            m.BuildMesh(or.Triangles, 9, true); //9 relevant decimal places,; include degenerate triangles

            //Check and remove eventual multiple triangles
            Console.WriteLine("Multiple triangles: " + m.DeleteMultipleTriangles().Count);
                      
            //Some statistics of the current mesh
            Console.WriteLine("Current mesh statistics:");
            Console.WriteLine("Number of vertices       : " + m.MeshVertices.Count);
            Console.WriteLine("Number of active vertices: " + m.GetActiveMeshVertices().Count);
            Console.WriteLine("Number of edges          : " + m.MeshEdges.Count);
            Console.WriteLine("Number of triangles      : " + m.MeshTriangles.Count);
            Console.WriteLine("Number of boundary edges : " + m.GetMeshBoundaryEdges().Count);
            Console.WriteLine("Is mesh consistent       : " + m.IsMeshConsistent());

            //Remove a single vertex; simply take the first vertex of the mesh. Removing a 
            //vertex also removes all triangles that share the removed vertex thus punching
            //a hole into the mesh. Note that in a general case, an arbitrary number of
            //triangles may be incident to a vertex.
            Console.WriteLine();
            m.DeleteVertex(0);
            Console.WriteLine("First vertex has been removed:");

            //Some statistics of the current mesh
            Console.WriteLine("Current mesh statistics:");
            Console.WriteLine("Number of vertices       : " + m.MeshVertices.Count);
            Console.WriteLine("Number of active vertices: " + m.GetActiveMeshVertices().Count);
            Console.WriteLine("Number of edges          : " + m.MeshEdges.Count);
            Console.WriteLine("Number of triangles      : " + m.MeshTriangles.Count);
            Console.WriteLine("Number of boundary edges : " + m.GetMeshBoundaryEdges().Count);
            Console.WriteLine("Is mesh consistent       : " + m.IsMeshConsistent());

            //Collapse an edge. This is, eighter move the start point of an edge to its end
            //point or move its end point to its start point. Anyway, this also removes two
            //triangles and one vertex in a general case.
            Console.WriteLine();
            m.CollapseEdge(m.MeshEdges.ConvertToList()[0], MeshEdge.CollapseDirection.ToEndPoint);
            Console.WriteLine("First edge has been collapsed:");
            
            //Some statistics of the current mesh
            Console.WriteLine("Current mesh statistics:");
            Console.WriteLine("Number of vertices       : " + m.MeshVertices.Count);
            Console.WriteLine("Number of active vertices: " + m.GetActiveMeshVertices().Count);
            Console.WriteLine("Number of edges          : " + m.MeshEdges.Count);
            Console.WriteLine("Number of triangles      : " + m.MeshTriangles.Count);
            Console.WriteLine("Number of boundary edges : " + m.GetMeshBoundaryEdges().Count);
            Console.WriteLine("Is mesh consistent       : " + m.IsMeshConsistent());

            //Perform a polygon reduction (LOD, level of detail) on the mesh.
            //The objective of LOD is to take a high-detail model with many polygons and to generate
            //a version using fewer polygons that looks reasonably similar to the original. The
            //reduction is done by collapsing edges. The edge collapse criterion follows a robust
            //heuristic proposed by Stan Melax (see http://www.melax.com/polychop). 
            Console.WriteLine();
            Console.WriteLine("Reduce mesh from " + m.MeshTriangles.Count + " to " + 
                (int)(0.5*m.MeshTriangles.Count) + ":");
            m.ReduceMesh((int)(0.5 * m.MeshTriangles.Count), true);

            //Some statistics of the current mesh
            Console.WriteLine("Current mesh statistics:");
            Console.WriteLine("Number of vertices       : " + m.MeshVertices.Count);
            Console.WriteLine("Number of active vertices: " + m.GetActiveMeshVertices().Count);
            Console.WriteLine("Number of edges          : " + m.MeshEdges.Count);
            Console.WriteLine("Number of triangles      : " + m.MeshTriangles.Count);
            Console.WriteLine("Number of boundary edges : " + m.GetMeshBoundaryEdges().Count);
            Console.WriteLine("Is mesh consistent       : " + m.IsMeshConsistent());
            
            Console.WriteLine();

            //---------------------------------------------------------------------
            // Visualize in dxf format
            // Get the free DWG TrueView viewer at www.autodesk.com to view dxf
            Scene sc = new Scene();
            Scene.Layer tlay = new Scene.Layer("reduced_surface", 5);
            Scene.Layer blay = new Scene.Layer("boundary_edges", 2);

            // Add objects to the scene
            sc.Add(m.ConvertMeshToTriangleList(), tlay);
            sc.Add(m.ConvertBoundariesToEdgeList(), blay);

            // Output the scene as dxf
            string outfile = @"..\..\reduced_surface.dxf";
            sc.WriteDxf(outfile);
            Console.WriteLine("dxf written to " + outfile);

            Console.WriteLine("Hit any key to continue.");
            Console.ReadLine();
        }
    }
}